//
//  FRMManager.m
//  AsReader 6Combo
//
//  Created by sps on 2020/10/27.
//

#import "RFMManager.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>

@interface RFMManager()
{
    int filterRSSI;
    int filterWet;
}
@end

@implementation RFMManager

#pragma mark - Public method
//----------------------------------------------------------------
-(void)setFilterRSSI:(int)rssi wet:(int)wet
{
    filterRSSI = rssi;
    filterWet = wet;
}

- (double) p_temperatureCalc:(uint8_t *)sensorData typeIndex:(int)typeIndex
{
    double result = 0;
    int offset = 0;
    int temp = 0;
    double tempCode_ = 0;
    double code1_ = 0;
    double code2_ = 0;
    
    double temp1_ = 0;
    double temp2_ = 0;
    
    offset = typeIndex;
    tempCode_ = sensorData[offset + 0] << 8 | (sensorData[offset + 1] & 0xFF);
    
    temp = sensorData[offset + 4] & 0xFF;
    temp = (temp << 4) + ((sensorData[offset + 5] >> 4) & 0x0F);
    code1_ = temp;
    
    temp = sensorData[offset + 5] & 0x0F;
    temp = (temp << 7) + ((sensorData[offset + 6] >> 1) & 0x7F);
    temp1_ = temp;
    
    temp = sensorData[offset + 6] & 0x01;
    temp = (temp << 8) + sensorData[offset + 7];
    temp = (temp << 3) + ((sensorData[offset + 8] >> 5) & 0x07);
    code2_ = temp;
    
    temp = sensorData[offset + 8] & 0x1F;
    temp = (temp << 6) + ((sensorData[offset + 9] >> 2) & 0x3F);
    temp2_ = temp;
    
    result = ((temp2_ - temp1_) / (code2_ - code1_) * (tempCode_ - code1_) + temp1_ - 800) / 10;
    
    return result;
}

- (RFMSensorItem *)parseData:(NSData *)pcEpc sensorData:(NSData *)sensorData rssi:(int8_t)rssi
{
    NSMutableString *tagEPC = [[NSMutableString alloc] init];
    NSMutableString *tagSensroData = [[NSMutableString alloc] init];
    
    uint8_t *pcEpcPtr = (uint8_t *)[pcEpc bytes];
    
    for(int i = 0; i < pcEpc.length; i++){
        [tagEPC appendFormat:@"%02X", *pcEpcPtr++ & 0xFF ];
    }
    
    uint8_t *sensPtr= (uint8_t *)[sensorData bytes];
    
    for(int i = 0; i < sensorData.length; i++){
        [tagSensroData appendFormat:@"%02X", *sensPtr++ & 0xFF ];
    }
    
    int sensorTypeindex = 6;
    int sensorCalcindex = 11;
    
    AsReaderInfo *info = [AsReaderInfo sharedInstance];
    if( [info.rfidModuleVersion containsString:@"RED4S"]==YES ){
        sensorTypeindex = 6;
        sensorCalcindex = 11;
    } else {
        sensorTypeindex = 0;
        sensorCalcindex = 3;
    }
    
    uint8_t *sensorBytes = (uint8_t *)[sensorData bytes];
    
    double tempValue = [self p_temperatureCalc:sensorBytes typeIndex:sensorCalcindex];
    
    NSString *temperature = [NSString stringWithFormat:@"%.1f",tempValue];
    
    NSString *cop = [tagSensroData substringWithRange:NSMakeRange(0, 4)];
    cop = [self p_parseCOPHexStringToIntSting:cop];
    
    NSString *cfqH = [tagSensroData substringWithRange:NSMakeRange(4, 4)];
    NSString *cfqL = [tagSensroData substringWithRange:NSMakeRange(8, 4)];
    NSString *cfq = [self p_parseCFQHexStringToIntStingH:cfqH stringL:cfqL];
    
    NSString *ct = [tagSensroData substringWithRange:NSMakeRange(12, 2)];
    ct = [self p_parseCTHexStringToIntSting:ct];
    
    NSString *cd = [tagSensroData substringFromIndex : 14];
    NSString *moisture = [cd substringWithRange:NSMakeRange(0, 4)];
    moisture = [NSString stringWithFormat:@"0x%@",moisture];
    NSString *onChipRSSI = [cd substringWithRange:NSMakeRange(4, 4)];
    
    RFMSensorItem *item = [RFMSensorItem new];
    item.pcEpc = tagEPC;
    item.sensorData = tagSensroData;
    
    item.cop = cop;
    item.cfq = cfq;
    item.ct = ct;
    item.cd = cd;
    
    item.temperature = temperature;
    item.moisture = moisture;
    item.rssi = onChipRSSI;
    
    unsigned int rssiIntVal = [self p_parseHexStringToInt:onChipRSSI];
    unsigned int wetIntVal  = [self p_parseHexStringToInt:moisture];
    
    NSLog(@"===> Filter %d  val: %d  ",filterRSSI,rssiIntVal);
    //Filter
    if(filterRSSI > rssiIntVal){
        NSLog(@"===> Filter %d ",rssiIntVal);
        return nil;
    }
    if(wetIntVal > filterWet) {
        item.moistureResult = @"WET";
    } else {
        item.moistureResult = @"DRY";
    }
    
    return item;
}

#pragma mark - Private method
//----------------------------------------------------------------
- (unsigned int)p_parseHexStringToInt:(NSString *)hexString
{
    unsigned int result = 0;
    NSScanner *scanner = [NSScanner scannerWithString:hexString];
    [scanner setScanLocation:0];
    [scanner scanHexInt:&result];
    return result;
}

- (NSString *)p_parseCOPHexStringToIntSting:(NSString *)hexString
{
    int result = [self p_parseHexStringToInt:hexString];
    return [NSString stringWithFormat:@"%.1f dBm",result/10.0];
}

- (NSString *)p_parseCFQHexStringToIntStingH:(NSString *)highString stringL:(NSString*)lowString
{
    int resultH = [self p_parseHexStringToInt:highString];
    int resultL = [self p_parseHexStringToInt:lowString];
    return [NSString stringWithFormat:@"%d.%d MHz",resultH,resultL];
}

- (NSString *)p_parseCTHexStringToIntSting:(NSString *)hexString
{
    int result = [self p_parseHexStringToInt:hexString];
    if(result == 0x03) {
        // Manus S3
        return  @"S3 Tag";
    } else if (result == 0x02) {
        // Manus S3
        return  @"S2 Tag";
    } else {
        return @"Unknown Tag";
    }
}






@end

