//  AboutController.m
//
#import "AboutController.h"
#import "ModelSettings.h"
#import "GlobalDefine.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>

@interface AboutController()<AsReaderRFIDDeviceDelegate, AsReaderDeviceDelegate>

@property (weak, nonatomic) IBOutlet UILabel *olAppVersion;
@property (weak, nonatomic) IBOutlet UILabel *olModel;
@property (weak, nonatomic) IBOutlet UILabel *olRegion;
@property (weak, nonatomic) IBOutlet UILabel *olBattery;
@property (weak, nonatomic) IBOutlet UILabel *olHWV;
@property (weak, nonatomic) IBOutlet UILabel *olFWV;
@property (weak, nonatomic) IBOutlet UILabel *olRFODModule;
@property (weak, nonatomic) IBOutlet UILabel *olAppInfo;
@end

@implementation AboutController

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    self.olAppVersion.text = [AsReaderRFIDDevice getSDKVersion];

    [AsReaderRFIDDevice sharedInstance].delegateDevice = self;
    [AsReaderRFIDDevice sharedInstance].delegateRFID = self;
 
    [self updateControl];
    [self.olAppInfo setText:[self appNameAndVersionNumberDisplayString]];
    
 
    if ( [[AsReaderRFIDDevice sharedInstance] getRegion]==YES ){
              [self performSelector:@selector(onTick) withObject:nil afterDelay:0.5f];
       }

}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [AsReaderRFIDDevice sharedInstance].delegateDevice = nil;
    [AsReaderRFIDDevice sharedInstance].delegateRFID = nil;
    
    [self setOlAppVersion:nil];
    [self setOlModel:nil];
    [self setOlHWV:nil];
    [self setOlRegion:nil];
    [self setOlBattery:nil];
    [self setOlFWV:nil];
}

#pragma mark - Private method
//----------------------------------------------------------------
- (void)updateControl
{
    dispatch_async(dispatch_get_main_queue(), ^{
        self.olHWV.text = [AsReaderInfo sharedInstance].deviceHardware;
        self.olFWV.text = [AsReaderInfo sharedInstance].deviceFirmware;
        self.olModel.text = [AsReaderInfo sharedInstance].deviceModelNumber;
   });
}

- (void) onTick
{
    //Get RFID Version Info
    [[AsReaderRFIDDevice sharedInstance] getRFIDModuleVersion];
}

/* APP version */
- (NSString *)appNameAndVersionNumberDisplayString
{
    NSDictionary *infoDictionary = [[NSBundle mainBundle] infoDictionary];
    NSString *majorVersion = [infoDictionary objectForKey:@"CFBundleShortVersionString"];
    NSString *minorVersion = [infoDictionary objectForKey:@"CFBundleVersion"];
    return [NSString stringWithFormat:@"%major :%@ (%@)", majorVersion, minorVersion];
}

#pragma mark - AsReaderRFIDDeviceDelegate
//----------------------------------------------------------------
- (void)batteryReceived:(int)battery
{
    dispatch_async(dispatch_get_main_queue(),^{
        self.olBattery.text = [NSString stringWithFormat:@"%d %%",battery];
    });
}

/* Region Infomation */
- (void)regionReceived:(int)region
{
    dispatch_async(dispatch_get_main_queue(), ^{
       NSString *strBand = @"";
        switch (region)
        {
            case REGION_KOREA_OLD:
            case REGION_KOREA:
                strBand = @"KOREA";
                break;
            case REGION_USA_OLD:
                strBand = @"US(OLD Type)";
                break;
            case REGION_USA:
                strBand = @"US1(North)";
                break;
            case REGION_USA2:
                strBand = @"US2";
                break;
            case REGION_EUROPE_OLD:
            case REGION_EUROPE:
                strBand = @"EU";
                break;
            case REGION_JAPAN_OLD:
            case REGION_JAPAN:
                strBand = @"JAPAN";
                break;
            case REGION_CHINA2_OLD:
            case REGION_CHINA1_OLD:
            case REGION_CHINA1:
            case REGION_CHINA2:
                strBand = @"CHINA";
                break;
            case REGION_AU_HK:
                strBand = @"ASIA";
                break;
            default:
                strBand = @"Unkown";
                break;
        }
       
       self.olRegion.text = strBand;
   });
}

/* RFID Module Version Infomation */
- (void)rfidModuleVersionReceived
{
    dispatch_async(dispatch_get_main_queue(), ^{
        AsReaderInfo  *info = [AsReaderInfo sharedInstance];
        self.olRFODModule.text = info.rfidModuleVersion;
    });
}

- (void)plugged:(BOOL)plug
{
    
}

- (void)readerConnected:(int)status
{
    
}

@end
