//  SettingsPowerController.m

#import "SettingsPowerController.h"
#import "ModelSettings.h"
#import "Utils.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>

@interface SettingsPowerController ()<AsReaderRFIDDeviceDelegate>
{
    NSIndexPath *_selectedIndexPath;
    int16_t powerLevel;
}

@property (nonatomic, strong)NSMutableArray *powerArray;
@property (nonatomic, assign)NSInteger initPower;
@end

@implementation SettingsPowerController

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    if( self.powerArray==nil ) {
        self.powerArray = [[NSMutableArray alloc] init];
    }
    
    NSLog(@"[POWER] Now %f  Min %f  Max %f ", [AsReaderInfo sharedInstance].rfidPower, [AsReaderInfo sharedInstance].rfidPowerMin, [AsReaderInfo sharedInstance].rfidPowerMax);
    int nMin = (int)[AsReaderInfo sharedInstance].rfidPowerMin;
    int nMAX = (int)[AsReaderInfo sharedInstance].rfidPowerMax;
    NSLog(@"[POWER Nu] Now %d  Min %d  Max %d ",(int)[AsReaderInfo sharedInstance].rfidPower, nMin, nMAX);
    
    for(int i = nMin; i <= nMAX; i = i + 5) {
        [self.powerArray addObject:[NSNumber numberWithInt:i]];
    }
    [self.tableView reloadData];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    powerLevel = [AsReaderInfo sharedInstance].rfidPower;
    [AsReaderRFIDDevice sharedInstance].delegateRFID = self;
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    [AsReaderRFIDDevice sharedInstance].delegateRFID = nil;
}

#pragma mark - Private method
//----------------------------------------------------------------
- (void) showAlert:(NSString *)strMsg
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [Utils showAlert:@"" message:strMsg view:self];
    });
}

#pragma mark - IBAction
//----------------------------------------------------------------
- (IBAction)rightBarButtonItemClicked:(id)sender
{
    [[AsReaderRFIDDevice sharedInstance] setOutputPowerLevel:powerLevel];
}

#pragma mark - AsReaderRFIDDeviceDelegate
//----------------------------------------------------------------
- (void)didSetOutputPowerLevel:(int)status
{
    if(status == 0) {
        //Sucess
        [self showAlert:@"Completed TX Power Set"];
    }
}

#pragma mark - UITableView datasource
//----------------------------------------------------------------
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

#pragma mark - UITableView delegate
//----------------------------------------------------------------
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    int ipower = [[self.powerArray objectAtIndex:indexPath.row] intValue];
    
    float power = (float) ipower / 10.0;
    cell.textLabel.text = [NSString stringWithFormat:@"%02.1f", power];
    
    if(powerLevel == ipower) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    int nCount = (int)self.powerArray.count;
    return nCount;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    powerLevel = [[self.powerArray objectAtIndex:indexPath.row] intValue];
    NSLog(@"select   %d ",powerLevel);
    [self.tableView reloadData];
}

@end
